@@ -5,6 +5,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"net/http"
+	"strings"
 	"testing"
 	"time"
 
@@ -32,6 +33,9 @@ func TestKubernetesClusters_ListClusters(t *testing.T) {
 			Status: &KubernetesClusterStatus{
 				State: KubernetesClusterStatusRunning,
 			},
+			RoutingAgent: &KubernetesRoutingAgent{
+				Enabled: PtrTo(true),
+			},
 			NodePools: []*KubernetesNodePool{
 				{
 					ID:    "1a17a012-cb31-4886-a787-deadbeef1191",
@@ -122,6 +126,9 @@ func TestKubernetesClusters_ListClusters(t *testing.T) {
 			"status": {
 				"state": "running"
 			},
+			"routing_agent": {
+				"enabled": true
+			},
 			"node_pools": [
 				{
 					"id": "1a17a012-cb31-4886-a787-deadbeef1191",
@@ -263,6 +270,9 @@ func TestKubernetesClusters_Get(t *testing.T) {
 		Status: &KubernetesClusterStatus{
 			State: KubernetesClusterStatusRunning,
 		},
+		RoutingAgent: &KubernetesRoutingAgent{
+			Enabled: PtrTo(true),
+		},
 		NodePools: []*KubernetesNodePool{
 			{
 				ID:    "deadbeef-dead-beef-dead-deadbeefb4b3",
@@ -314,6 +324,9 @@ func TestKubernetesClusters_Get(t *testing.T) {
 		"status": {
 			"state": "running"
 		},
+		"routing_agent": {
+			"enabled": true
+		},
 		"node_pools": [
 			{
 				"id": "deadbeef-dead-beef-dead-deadbeefb4b3",
@@ -351,7 +364,7 @@ func TestKubernetesClusters_Get(t *testing.T) {
 		],
 		"maintenance_policy": {
 			"start_time": "00:00",
-			"day": "monday"
+			"day": "Monday"
 		},
 		"created_at": "2018-06-15T07:10:23Z",
 		"updated_at": "2018-06-15T07:11:26Z"
@@ -384,6 +397,7 @@ func TestKubernetesClusters_GetUser(t *testing.T) {
 	kubeSvc := client.Kubernetes
 	want := &KubernetesClusterUser{
 		Username: "foo@example.com",
+		ID:       "deadbeef-dead-beef-dead-deadbeef05e8",
 		Groups: []string{
 			"foo:bar",
 		},
@@ -392,6 +406,7 @@ func TestKubernetesClusters_GetUser(t *testing.T) {
 {
 	"kubernetes_cluster_user": {
 		"username": "foo@example.com",
+		"id": "deadbeef-dead-beef-dead-deadbeef05e8",
 		"groups": ["foo:bar"]
 	}
 }`
@@ -543,6 +558,8 @@ func TestKubernetesClusters_Create(t *testing.T) {
 
 	kubeSvc := client.Kubernetes
 	enabled := true
+	scaleDownUtilizationThreshold := 0.5
+	scaleDownUnneededTime := "1m30s"
 
 	want := &KubernetesCluster{
 		ID:            "8d91899c-0739-4a1a-acc5-deadbeefbb8f",
@@ -555,6 +572,9 @@ func TestKubernetesClusters_Create(t *testing.T) {
 		VPCUUID:       "880b7f98-f062-404d-b33c-458d545696f6",
 		HA:            true,
 		SurgeUpgrade:  true,
+		RoutingAgent: &KubernetesRoutingAgent{
+			Enabled: PtrTo(true),
+		},
 		NodePools: []*KubernetesNodePool{
 			{
 				ID:     "8d91899c-0739-4a1a-acc5-deadbeefbb8a",
@@ -575,6 +595,10 @@ func TestKubernetesClusters_Create(t *testing.T) {
 				"1.2.3.4/32",
 			},
 		},
+		ClusterAutoscalerConfiguration: &KubernetesClusterAutoscalerConfiguration{
+			ScaleDownUtilizationThreshold: &scaleDownUtilizationThreshold,
+			ScaleDownUnneededTime:         &scaleDownUnneededTime,
+		},
 	}
 	createRequest := &KubernetesClusterCreateRequest{
 		Name:          want.Name,
@@ -586,6 +610,9 @@ func TestKubernetesClusters_Create(t *testing.T) {
 		ServiceSubnet: want.ServiceSubnet,
 		SurgeUpgrade:  true,
 		HA:            true,
+		RoutingAgent: &KubernetesRoutingAgent{
+			Enabled: PtrTo(true),
+		},
 		NodePools: []*KubernetesNodePoolCreateRequest{
 			{
 				Size:      want.NodePools[0].Size,
@@ -598,7 +625,8 @@ func TestKubernetesClusters_Create(t *testing.T) {
 				MaxNodes:  want.NodePools[0].MaxNodes,
 			},
 		},
-		MaintenancePolicy: want.MaintenancePolicy,
+		MaintenancePolicy:              want.MaintenancePolicy,
+		ClusterAutoscalerConfiguration: want.ClusterAutoscalerConfiguration,
 	}
 
 	jBlob := `
@@ -617,6 +645,9 @@ func TestKubernetesClusters_Create(t *testing.T) {
 		"vpc_uuid": "880b7f98-f062-404d-b33c-458d545696f6",
 		"ha": true,
 		"surge_upgrade": true,
+		"routing_agent": {
+			"enabled": true
+		},
 		"node_pools": [
 			{
 				"id": "8d91899c-0739-4a1a-acc5-deadbeefbb8a",
@@ -633,14 +664,18 @@ func TestKubernetesClusters_Create(t *testing.T) {
 		],
 		"maintenance_policy": {
 			"start_time": "00:00",
-			"day": "monday"
+			"day": "Monday"
 		},
-        "control_plane_firewall": {
-             "enabled": true,
-             "allowed_addresses": [
-                 "1.2.3.4/32"
-             ]
-        }
+		"control_plane_firewall": {
+			"enabled": true,
+			"allowed_addresses": [
+					"1.2.3.4/32"
+			]
+		},
+		"cluster_autoscaler_configuration": {
+      "scale_down_utilization_threshold": 0.5,
+      "scale_down_unneeded_time": "1m30s"
+    }
 	}
 }`
 
@@ -743,7 +778,7 @@ func TestKubernetesClusters_Create_AutoScalePool(t *testing.T) {
 		],
 		"maintenance_policy": {
 			"start_time": "00:00",
-			"day": "monday"
+			"day": "Monday"
 		}
 	}
 }`
@@ -771,6 +806,8 @@ func TestKubernetesClusters_Update(t *testing.T) {
 
 	kubeSvc := client.Kubernetes
 	enabled := true
+	scaleDownUtilizationThreshold := 0.2
+	scaleDownUnneededTime := "1m27s"
 
 	want := &KubernetesCluster{
 		ID:            "8d91899c-0739-4a1a-acc5-deadbeefbb8f",
@@ -805,6 +842,13 @@ func TestKubernetesClusters_Update(t *testing.T) {
 				"1.2.3.4/32",
 			},
 		},
+		ClusterAutoscalerConfiguration: &KubernetesClusterAutoscalerConfiguration{
+			ScaleDownUtilizationThreshold: &scaleDownUtilizationThreshold,
+			ScaleDownUnneededTime:         &scaleDownUnneededTime,
+		},
+		RoutingAgent: &KubernetesRoutingAgent{
+			Enabled: PtrTo(true),
+		},
 	}
 	updateRequest := &KubernetesClusterUpdateRequest{
 		Name:              want.Name,
@@ -817,6 +861,13 @@ func TestKubernetesClusters_Update(t *testing.T) {
 				"1.2.3.4/32",
 			},
 		},
+		ClusterAutoscalerConfiguration: &KubernetesClusterAutoscalerConfiguration{
+			ScaleDownUtilizationThreshold: &scaleDownUtilizationThreshold,
+			ScaleDownUnneededTime:         &scaleDownUnneededTime,
+		},
+		RoutingAgent: &KubernetesRoutingAgent{
+			Enabled: PtrTo(true),
+		},
 	}
 
 	jBlob := `
@@ -851,18 +902,25 @@ func TestKubernetesClusters_Update(t *testing.T) {
 		],
 		"maintenance_policy": {
 			"start_time": "00:00",
-			"day": "monday"
+			"day": "Monday"
 		},
 		"control_plane_firewall": {
-             "enabled": true,
-             "allowed_addresses": [
-                 "1.2.3.4/32"
-             ]
-        }
+			"enabled": true,
+			"allowed_addresses": [
+					"1.2.3.4/32"
+			]
+		},	
+		"cluster_autoscaler_configuration": {
+      		"scale_down_utilization_threshold": 0.2,
+      		"scale_down_unneeded_time": "1m27s"
+    	},
+		"routing_agent": {
+			"enabled": true
+		}
 	}
 }`
 
-	expectedReqJSON := `{"name":"antoine-test-cluster","tags":["cluster-tag-1","cluster-tag-2"],"maintenance_policy":{"start_time":"00:00","duration":"","day":"monday"},"surge_upgrade":true,"control_plane_firewall":{"enabled":true,"allowed_addresses":["1.2.3.4/32"]}}
+	expectedReqJSON := `{"name":"antoine-test-cluster","tags":["cluster-tag-1","cluster-tag-2"],"maintenance_policy":{"start_time":"00:00","duration":"","day":"monday"},"surge_upgrade":true,"control_plane_firewall":{"enabled":true,"allowed_addresses":["1.2.3.4/32"]},"cluster_autoscaler_configuration":{"scale_down_utilization_threshold":0.2,"scale_down_unneeded_time":"1m27s"},"routing_agent":{"enabled":true}}
 `
 
 	mux.HandleFunc("/v2/kubernetes/clusters/8d91899c-0739-4a1a-acc5-deadbeefbb8f", func(w http.ResponseWriter, r *http.Request) {
@@ -944,7 +1002,7 @@ func TestKubernetesClusters_Update_FalseAutoUpgrade(t *testing.T) {
 		],
 		"maintenance_policy": {
 			"start_time": "00:00",
-			"day": "monday"
+			"day": "Monday"
 		}
 	}
 }`
@@ -1271,6 +1329,64 @@ func TestKubernetesClusters_GetNodePool(t *testing.T) {
 	require.Equal(t, want, got)
 }
 
+func TestKubernetesClusters_GetNodePoolTemplate(t *testing.T) {
+	setup()
+	defer teardown()
+	kubeSvc := client.Kubernetes
+	want := &KubernetesNodePoolTemplate{
+		Template: &KubernetesNodeTemplate{
+			ClusterUUID: "8d91899c-0739-4a1a-acc5-deadbeefbb8a",
+			Name:        "pool-a",
+			Slug:        "s-1vcpu-2gb",
+			Taints:      []string{"some-key=some-value:NoSchedule"},
+			Labels: map[string]string{
+				"some-label": "some-value",
+			},
+			Capacity: &KubernetesNodePoolResources{
+				CPU:    1,
+				Memory: "2048Mi",
+				Pods:   110,
+			},
+			Allocatable: &KubernetesNodePoolResources{
+				CPU:    390,
+				Memory: "1024Mi",
+				Pods:   110,
+			},
+		},
+	}
+	jBlob := `
+{
+  "template": {
+    "cluster_uuid": "8d91899c-0739-4a1a-acc5-deadbeefbb8a",
+    "name": "pool-a",
+    "slug": "s-1vcpu-2gb",
+    "labels": {
+      "some-label": "some-value"
+    },
+    "taints": ["some-key=some-value:NoSchedule"],
+    "capacity": {
+      "cpu": 1,
+      "memory": "2048Mi",
+      "pods": 110
+    },
+    "allocatable": {
+      "cpu": 390,
+      "memory": "1024Mi",
+      "pods": 110
+    }
+  }
+}
+`
+	mux.HandleFunc("/v2/kubernetes/clusters/8d91899c-0739-4a1a-acc5-deadbeefbb8a/node_pools_template/pool-a", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, http.MethodGet)
+		fmt.Fprint(w, jBlob)
+	})
+	got, _, err := kubeSvc.GetNodePoolTemplate(ctx, "8d91899c-0739-4a1a-acc5-deadbeefbb8a", "pool-a")
+	require.NoError(t, err)
+	require.Equal(t, want, got)
+
+}
+
 func TestKubernetesClusters_ListNodePools(t *testing.T) {
 	setup()
 	defer teardown()
@@ -1778,6 +1894,47 @@ func TestKubernetesRunClusterlint_WithoutRequestBody(t *testing.T) {
 
 }
 
+func TestKubernetesClusterStatusMessages(t *testing.T) {
+	setup()
+	defer teardown()
+
+	since := time.Date(2019, 10, 30, 5, 34, 7, 0, time.UTC)
+	kubeSvc := client.Kubernetes
+
+	jBlob := `
+{
+	"messages": [
+		{
+			"message": "droplet limit exceeded, please contact support for an increase",
+			"timestamp": "2019-10-31T00:00:00Z"
+		}
+	]
+}`
+
+	want := []*KubernetesClusterStatusMessage{
+		{
+			Message:   "droplet limit exceeded, please contact support for an increase",
+			Timestamp: time.Date(2019, 10, 31, 0, 0, 0, 0, time.UTC),
+		},
+	}
+
+	mux.HandleFunc("/v2/kubernetes/clusters/8d91899c-0739-4a1a-acc5-deadbeefbb8f/status_messages", func(w http.ResponseWriter, r *http.Request) {
+		testMethod(t, r, http.MethodGet)
+		// urls escape the colon character as %3A
+		sinceFormatted := since.Format(time.RFC3339)
+		urlEncodedSince := strings.ReplaceAll(sinceFormatted, ":", "%3A")
+
+		require.Equal(t, "since="+urlEncodedSince, r.URL.Query().Encode())
+		fmt.Fprint(w, jBlob)
+	})
+
+	messages, _, err := kubeSvc.GetClusterStatusMessages(ctx, "8d91899c-0739-4a1a-acc5-deadbeefbb8f", &KubernetesGetClusterStatusMessagesRequest{
+		Since: &since,
+	})
+	require.NoError(t, err)
+	assert.Equal(t, want, messages)
+}
+
 func TestKubernetesGetClusterlint_WithRunID(t *testing.T) {
 	setup()
 	defer teardown()
