@@ -171,9 +171,13 @@ type PollOpts struct {
 }
 
 // WithPollOpts configures a Client to use the specified options when polling from the API.
+//
+// If [PollOpts.BackoffFunc] is nil, the existing backoff function will be preserved.
 func WithPollOpts(opts PollOpts) ClientOption {
 	return func(client *Client) {
-		client.pollBackoffFunc = opts.BackoffFunc
+		if opts.BackoffFunc != nil {
+			client.pollBackoffFunc = opts.BackoffFunc
+		}
 	}
 }
 
@@ -195,9 +199,13 @@ type RetryOpts struct {
 
 // WithRetryOpts configures a Client to use the specified options when retrying API
 // requests.
+//
+// If [RetryOpts.BackoffFunc] is nil, the existing backoff function will be preserved.
 func WithRetryOpts(opts RetryOpts) ClientOption {
 	return func(client *Client) {
-		client.retryBackoffFunc = opts.BackoffFunc
+		if opts.BackoffFunc != nil {
+			client.retryBackoffFunc = opts.BackoffFunc
+		}
 		client.retryMaxRetries = opts.MaxRetries
 	}
 }
@@ -259,7 +267,7 @@ func NewClient(options ...ClientOption) *Client {
 	client.buildUserAgent()
 	if client.instrumentationRegistry != nil {
 		i := instrumentation.New("api", client.instrumentationRegistry)
-		client.httpClient.Transport = i.InstrumentedRoundTripper()
+		client.httpClient.Transport = i.InstrumentedRoundTripper(client.httpClient.Transport)
 	}
 
 	client.handler = assembleHandlerChain(client)
@@ -317,22 +325,6 @@ func (c *Client) Do(req *http.Request, v any) (*Response, error) {
 	return c.handler.Do(req, v)
 }
 
-func (c *Client) all(f func(int) (*Response, error)) error {
-	var (
-		page = 1
-	)
-	for {
-		resp, err := f(page)
-		if err != nil {
-			return err
-		}
-		if resp.Meta.Pagination == nil || resp.Meta.Pagination.NextPage == 0 {
-			return nil
-		}
-		page = resp.Meta.Pagination.NextPage
-	}
-}
-
 func (c *Client) buildUserAgent() {
 	switch {
 	case c.applicationName != "" && c.applicationVersion != "":
