@@ -1,17 +1,14 @@
 package hcloud
 
 import (
-	"bytes"
 	"context"
-	"encoding/json"
-	"errors"
 	"fmt"
 	"net"
-	"net/http"
 	"net/url"
 	"strconv"
 	"time"
 
+	"github.com/hetznercloud/hcloud-go/v2/hcloud/exp/ctxutil"
 	"github.com/hetznercloud/hcloud-go/v2/hcloud/schema"
 )
 
@@ -155,26 +152,21 @@ const (
 // changeDNSPtr changes or resets the reverse DNS pointer for a IP address.
 // Pass a nil ptr to reset the reverse DNS pointer to its default value.
 func (s *Server) changeDNSPtr(ctx context.Context, client *Client, ip net.IP, ptr *string) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/change_dns_ptr"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, s.ID)
+
 	reqBody := schema.ServerActionChangeDNSPtrRequest{
 		IP:     ip.String(),
 		DNSPtr: ptr,
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/change_dns_ptr", s.ID)
-	req, err := client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionChangeDNSPtrResponse{}
-	resp, err := client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionChangeDNSPtrResponse](ctx, client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -198,44 +190,33 @@ type ServerClient struct {
 
 // GetByID retrieves a server by its ID. If the server does not exist, nil is returned.
 func (c *ServerClient) GetByID(ctx context.Context, id int64) (*Server, *Response, error) {
-	req, err := c.client.NewRequest(ctx, "GET", fmt.Sprintf("/servers/%d", id), nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	var body schema.ServerGetResponse
-	resp, err := c.client.Do(req, &body)
+	reqPath := fmt.Sprintf(opPath, id)
+
+	respBody, resp, err := getRequest[schema.ServerGetResponse](ctx, c.client, reqPath)
 	if err != nil {
 		if IsError(err, ErrorCodeNotFound) {
 			return nil, resp, nil
 		}
-		return nil, nil, err
+		return nil, resp, err
 	}
-	return ServerFromSchema(body.Server), resp, nil
+
+	return ServerFromSchema(respBody.Server), resp, nil
 }
 
 // GetByName retrieves a server by its name. If the server does not exist, nil is returned.
 func (c *ServerClient) GetByName(ctx context.Context, name string) (*Server, *Response, error) {
-	if name == "" {
-		return nil, nil, nil
-	}
-	servers, response, err := c.List(ctx, ServerListOpts{Name: name})
-	if len(servers) == 0 {
-		return nil, response, err
-	}
-	return servers[0], response, err
+	return firstByName(name, func() ([]*Server, *Response, error) {
+		return c.List(ctx, ServerListOpts{Name: name})
+	})
 }
 
 // Get retrieves a server by its ID if the input can be parsed as an integer, otherwise it
 // retrieves a server by its name. If the server does not exist, nil is returned.
 func (c *ServerClient) Get(ctx context.Context, idOrName string) (*Server, *Response, error) {
-	if id, err := strconv.ParseInt(idOrName, 10, 64); err == nil {
-		srv, res, err := c.GetByID(ctx, id)
-		if srv != nil || err != nil {
-			return srv, res, err
-		}
-	}
-	return c.GetByName(ctx, idOrName)
+	return getByIDOrName(ctx, c.GetByID, c.GetByName, idOrName)
 }
 
 // ServerListOpts specifies options for listing servers.
@@ -265,22 +246,17 @@ func (l ServerListOpts) values() url.Values {
 // Please note that filters specified in opts are not taken into account
 // when their value corresponds to their zero value or when they are empty.
 func (c *ServerClient) List(ctx context.Context, opts ServerListOpts) ([]*Server, *Response, error) {
-	path := "/servers?" + opts.values().Encode()
-	req, err := c.client.NewRequest(ctx, "GET", path, nil)
+	const opPath = "/servers?%s"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, opts.values().Encode())
+
+	respBody, resp, err := getRequest[schema.ServerListResponse](ctx, c.client, reqPath)
 	if err != nil {
-		return nil, nil, err
+		return nil, resp, err
 	}
 
-	var body schema.ServerListResponse
-	resp, err := c.client.Do(req, &body)
-	if err != nil {
-		return nil, nil, err
-	}
-	servers := make([]*Server, 0, len(body.Servers))
-	for _, s := range body.Servers {
-		servers = append(servers, ServerFromSchema(s))
-	}
-	return servers, resp, nil
+	return allFromSchemaFunc(respBody.Servers, ServerFromSchema), resp, nil
 }
 
 // All returns all servers.
@@ -290,22 +266,10 @@ func (c *ServerClient) All(ctx context.Context) ([]*Server, error) {
 
 // AllWithOpts returns all servers for the given options.
 func (c *ServerClient) AllWithOpts(ctx context.Context, opts ServerListOpts) ([]*Server, error) {
-	allServers := []*Server{}
-
-	err := c.client.all(func(page int) (*Response, error) {
+	return iterPages(func(page int) ([]*Server, *Response, error) {
 		opts.Page = page
-		servers, resp, err := c.List(ctx, opts)
-		if err != nil {
-			return resp, err
-		}
-		allServers = append(allServers, servers...)
-		return resp, nil
+		return c.List(ctx, opts)
 	})
-	if err != nil {
-		return nil, err
-	}
-
-	return allServers, nil
 }
 
 // ServerCreateOpts specifies options for creating a new server.
@@ -342,22 +306,16 @@ type ServerCreateFirewall struct {
 // Validate checks if options are valid.
 func (o ServerCreateOpts) Validate() error {
 	if o.Name == "" {
-		return errors.New("missing name")
+		return missingField(o, "Name")
 	}
 	if o.ServerType == nil || (o.ServerType.ID == 0 && o.ServerType.Name == "") {
-		return errors.New("missing server type")
+		return missingField(o, "ServerType")
 	}
 	if o.Image == nil || (o.Image.ID == 0 && o.Image.Name == "") {
-		return errors.New("missing image")
+		return missingField(o, "Image")
 	}
 	if o.Location != nil && o.Datacenter != nil {
-		return errors.New("location and datacenter are mutually exclusive")
-	}
-	if o.PublicNet != nil {
-		if !o.PublicNet.EnableIPv4 && !o.PublicNet.EnableIPv6 &&
-			len(o.Networks) == 0 && (o.StartAfterCreate == nil || *o.StartAfterCreate) {
-			return errors.New("missing networks or StartAfterCreate == false when EnableIPv4 and EnableIPv6 is false")
-		}
+		return mutuallyExclusiveFields(o, "Location", "Datacenter")
 	}
 	return nil
 }
@@ -372,8 +330,15 @@ type ServerCreateResult struct {
 
 // Create creates a new server.
 func (c *ServerClient) Create(ctx context.Context, opts ServerCreateOpts) (ServerCreateResult, *Response, error) {
+	const opPath = "/servers"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerCreateResult{}
+
+	reqPath := opPath
+
 	if err := opts.Validate(); err != nil {
-		return ServerCreateResult{}, nil, err
+		return result, nil, err
 	}
 
 	var reqBody schema.ServerCreateRequest
@@ -434,29 +399,19 @@ func (c *ServerClient) Create(ctx context.Context, opts ServerCreateOpts) (Serve
 	if opts.PlacementGroup != nil {
 		reqBody.PlacementGroup = opts.PlacementGroup.ID
 	}
-	reqBodyData, err := json.Marshal(reqBody)
+
+	respBody, resp, err := postRequest[schema.ServerCreateResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
-		return ServerCreateResult{}, nil, err
+		return result, resp, err
 	}
 
-	req, err := c.client.NewRequest(ctx, "POST", "/servers", bytes.NewReader(reqBodyData))
-	if err != nil {
-		return ServerCreateResult{}, nil, err
-	}
-
-	var respBody schema.ServerCreateResponse
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerCreateResult{}, resp, err
-	}
-	result := ServerCreateResult{
-		Server:      ServerFromSchema(respBody.Server),
-		Action:      ActionFromSchema(respBody.Action),
-		NextActions: ActionsFromSchema(respBody.NextActions),
-	}
+	result.Server = ServerFromSchema(respBody.Server)
+	result.Action = ActionFromSchema(respBody.Action)
+	result.NextActions = ActionsFromSchema(respBody.NextActions)
 	if respBody.RootPassword != nil {
 		result.RootPassword = *respBody.RootPassword
 	}
+
 	return result, resp, nil
 }
 
@@ -475,20 +430,21 @@ func (c *ServerClient) Delete(ctx context.Context, server *Server) (*Response, e
 
 // DeleteWithResult deletes a server and returns the parsed response containing the action.
 func (c *ServerClient) DeleteWithResult(ctx context.Context, server *Server) (*ServerDeleteResult, *Response, error) {
-	req, err := c.client.NewRequest(ctx, "DELETE", fmt.Sprintf("/servers/%d", server.ID), nil)
+	const opPath = "/servers/%d"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := &ServerDeleteResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := deleteRequest[schema.ServerDeleteResponse](ctx, c.client, reqPath)
 	if err != nil {
-		return &ServerDeleteResult{}, nil, err
+		return result, resp, err
 	}
 
-	var respBody schema.ServerDeleteResponse
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return &ServerDeleteResult{}, resp, err
-	}
+	result.Action = ActionFromSchema(respBody.Action)
 
-	return &ServerDeleteResult{
-		Action: ActionFromSchema(respBody.Action),
-	}, resp, nil
+	return result, resp, nil
 }
 
 // ServerUpdateOpts specifies options for updating a server.
@@ -499,108 +455,98 @@ type ServerUpdateOpts struct {
 
 // Update updates a server.
 func (c *ServerClient) Update(ctx context.Context, server *Server, opts ServerUpdateOpts) (*Server, *Response, error) {
+	const opPath = "/servers/%d"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerUpdateRequest{
 		Name: opts.Name,
 	}
 	if opts.Labels != nil {
 		reqBody.Labels = &opts.Labels
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d", server.ID)
-	req, err := c.client.NewRequest(ctx, "PUT", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerUpdateResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := putRequest[schema.ServerUpdateResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ServerFromSchema(respBody.Server), resp, nil
 }
 
 // Poweron starts a server.
 func (c *ServerClient) Poweron(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/poweron", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/poweron"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionPoweronResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionPoweronResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // Reboot reboots a server.
 func (c *ServerClient) Reboot(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/reboot", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/reboot"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionRebootResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionRebootResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // Reset resets a server.
 func (c *ServerClient) Reset(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/reset", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/reset"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionResetResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionResetResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // Shutdown shuts down a server.
 func (c *ServerClient) Shutdown(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/shutdown", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/shutdown"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionShutdownResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionShutdownResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // Poweroff stops a server.
 func (c *ServerClient) Poweroff(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/poweroff", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/poweroff"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionPoweroffResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionPoweroffResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -612,21 +558,22 @@ type ServerResetPasswordResult struct {
 
 // ResetPassword resets a server's password.
 func (c *ServerClient) ResetPassword(ctx context.Context, server *Server) (ServerResetPasswordResult, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/reset_password", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
+	const opPath = "/servers/%d/actions/reset_password"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerResetPasswordResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionResetPasswordResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
-		return ServerResetPasswordResult{}, nil, err
+		return result, resp, err
 	}
 
-	respBody := schema.ServerActionResetPasswordResponse{}
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerResetPasswordResult{}, resp, err
-	}
-	return ServerResetPasswordResult{
-		Action:       ActionFromSchema(respBody.Action),
-		RootPassword: respBody.RootPassword,
-	}, resp, nil
+	result.Action = ActionFromSchema(respBody.Action)
+	result.RootPassword = respBody.RootPassword
+
+	return result, resp, nil
 }
 
 // ServerCreateImageOpts specifies options for creating an image from a server.
@@ -644,7 +591,7 @@ func (o ServerCreateImageOpts) Validate() error {
 	case "":
 		break
 	default:
-		return errors.New("invalid type")
+		return invalidFieldValue(o, "Type", o.Type)
 	}
 
 	return nil
@@ -658,10 +605,17 @@ type ServerCreateImageResult struct {
 
 // CreateImage creates an image from a server.
 func (c *ServerClient) CreateImage(ctx context.Context, server *Server, opts *ServerCreateImageOpts) (ServerCreateImageResult, *Response, error) {
-	var reqBody schema.ServerActionCreateImageRequest
+	const opPath = "/servers/%d/actions/create_image"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerCreateImageResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	reqBody := schema.ServerActionCreateImageRequest{}
 	if opts != nil {
 		if err := opts.Validate(); err != nil {
-			return ServerCreateImageResult{}, nil, fmt.Errorf("invalid options: %s", err)
+			return result, nil, err
 		}
 		if opts.Description != nil {
 			reqBody.Description = opts.Description
@@ -673,26 +627,16 @@ func (c *ServerClient) CreateImage(ctx context.Context, server *Server, opts *Se
 			reqBody.Labels = &opts.Labels
 		}
 	}
-	reqBodyData, err := json.Marshal(reqBody)
+
+	respBody, resp, err := postRequest[schema.ServerActionCreateImageResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
-		return ServerCreateImageResult{}, nil, err
+		return result, resp, err
 	}
 
-	path := fmt.Sprintf("/servers/%d/actions/create_image", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return ServerCreateImageResult{}, nil, err
-	}
+	result.Image = ImageFromSchema(respBody.Image)
+	result.Action = ActionFromSchema(respBody.Action)
 
-	respBody := schema.ServerActionCreateImageResponse{}
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerCreateImageResult{}, resp, err
-	}
-	return ServerCreateImageResult{
-		Action: ActionFromSchema(respBody.Action),
-		Image:  ImageFromSchema(respBody.Image),
-	}, resp, nil
+	return result, resp, nil
 }
 
 // ServerEnableRescueOpts specifies options for enabling rescue mode for a server.
@@ -709,48 +653,43 @@ type ServerEnableRescueResult struct {
 
 // EnableRescue enables rescue mode for a server.
 func (c *ServerClient) EnableRescue(ctx context.Context, server *Server, opts ServerEnableRescueOpts) (ServerEnableRescueResult, *Response, error) {
+	const opPath = "/servers/%d/actions/enable_rescue"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerEnableRescueResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionEnableRescueRequest{
 		Type: Ptr(string(opts.Type)),
 	}
 	for _, sshKey := range opts.SSHKeys {
 		reqBody.SSHKeys = append(reqBody.SSHKeys, sshKey.ID)
 	}
-	reqBodyData, err := json.Marshal(reqBody)
+
+	respBody, resp, err := postRequest[schema.ServerActionEnableRescueResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
-		return ServerEnableRescueResult{}, nil, err
+		return result, resp, err
 	}
 
-	path := fmt.Sprintf("/servers/%d/actions/enable_rescue", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return ServerEnableRescueResult{}, nil, err
-	}
+	result.Action = ActionFromSchema(respBody.Action)
+	result.RootPassword = respBody.RootPassword
 
-	respBody := schema.ServerActionEnableRescueResponse{}
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerEnableRescueResult{}, resp, err
-	}
-	result := ServerEnableRescueResult{
-		Action:       ActionFromSchema(respBody.Action),
-		RootPassword: respBody.RootPassword,
-	}
 	return result, resp, nil
 }
 
 // DisableRescue disables rescue mode for a server.
 func (c *ServerClient) DisableRescue(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/disable_rescue", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/disable_rescue"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionDisableRescueResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionDisableRescueResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -776,30 +715,24 @@ func (c *ServerClient) Rebuild(ctx context.Context, server *Server, opts ServerR
 
 // RebuildWithResult rebuilds a server.
 func (c *ServerClient) RebuildWithResult(ctx context.Context, server *Server, opts ServerRebuildOpts) (ServerRebuildResult, *Response, error) {
+	const opPath = "/servers/%d/actions/rebuild"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerRebuildResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionRebuildRequest{}
 	if opts.Image.ID != 0 || opts.Image.Name != "" {
 		reqBody.Image = schema.IDOrName{ID: opts.Image.ID, Name: opts.Image.Name}
 	}
-	reqBodyData, err := json.Marshal(reqBody)
+
+	respBody, resp, err := postRequest[schema.ServerActionRebuildResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
-		return ServerRebuildResult{}, nil, err
+		return result, resp, err
 	}
 
-	path := fmt.Sprintf("/servers/%d/actions/rebuild", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return ServerRebuildResult{}, nil, err
-	}
-
-	respBody := schema.ServerActionRebuildResponse{}
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerRebuildResult{}, resp, err
-	}
-
-	result := ServerRebuildResult{
-		Action: ActionFromSchema(respBody.Action),
-	}
+	result.Action = ActionFromSchema(respBody.Action)
 	if respBody.RootPassword != nil {
 		result.RootPassword = *respBody.RootPassword
 	}
@@ -809,42 +742,36 @@ func (c *ServerClient) RebuildWithResult(ctx context.Context, server *Server, op
 
 // AttachISO attaches an ISO to a server.
 func (c *ServerClient) AttachISO(ctx context.Context, server *Server, iso *ISO) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/attach_iso"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionAttachISORequest{}
 	if iso.ID != 0 || iso.Name != "" {
 		reqBody.ISO = schema.IDOrName{ID: iso.ID, Name: iso.Name}
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/attach_iso", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionAttachISOResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionAttachISOResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // DetachISO detaches the currently attached ISO from a server.
 func (c *ServerClient) DetachISO(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/detach_iso", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/detach_iso"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionDetachISOResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionDetachISOResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -853,33 +780,31 @@ func (c *ServerClient) DetachISO(ctx context.Context, server *Server) (*Action,
 func (c *ServerClient) EnableBackup(ctx context.Context, server *Server, window string) (*Action, *Response, error) {
 	_ = window
 
-	path := fmt.Sprintf("/servers/%d/actions/enable_backup", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/enable_backup"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionEnableBackupResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionEnableBackupResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // DisableBackup disables backup for a server.
 func (c *ServerClient) DisableBackup(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/disable_backup", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/disable_backup"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionDisableBackupResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionDisableBackupResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -891,28 +816,23 @@ type ServerChangeTypeOpts struct {
 
 // ChangeType changes a server's type.
 func (c *ServerClient) ChangeType(ctx context.Context, server *Server, opts ServerChangeTypeOpts) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/change_type"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionChangeTypeRequest{
 		UpgradeDisk: opts.UpgradeDisk,
 	}
 	if opts.ServerType.ID != 0 || opts.ServerType.Name != "" {
 		reqBody.ServerType = schema.IDOrName{ID: opts.ServerType.ID, Name: opts.ServerType.Name}
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/change_type", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionChangeTypeResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionChangeTypeResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, nil
 }
 
@@ -934,27 +854,22 @@ type ServerChangeProtectionOpts struct {
 
 // ChangeProtection changes the resource protection level of a server.
 func (c *ServerClient) ChangeProtection(ctx context.Context, server *Server, opts ServerChangeProtectionOpts) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/change_protection"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionChangeProtectionRequest{
 		Rebuild: opts.Rebuild,
 		Delete:  opts.Delete,
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/change_protection", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionChangeProtectionResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionChangeProtectionResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
-	return ActionFromSchema(respBody.Action), resp, err
+
+	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 // ServerRequestConsoleResult is the result of requesting a WebSocket VNC console.
@@ -966,22 +881,23 @@ type ServerRequestConsoleResult struct {
 
 // RequestConsole requests a WebSocket VNC console.
 func (c *ServerClient) RequestConsole(ctx context.Context, server *Server) (ServerRequestConsoleResult, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/request_console", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
+	const opPath = "/servers/%d/actions/request_console"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	result := ServerRequestConsoleResult{}
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionRequestConsoleResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
-		return ServerRequestConsoleResult{}, nil, err
+		return result, resp, err
 	}
 
-	respBody := schema.ServerActionRequestConsoleResponse{}
-	resp, err := c.client.Do(req, &respBody)
-	if err != nil {
-		return ServerRequestConsoleResult{}, resp, err
-	}
-	return ServerRequestConsoleResult{
-		Action:   ActionFromSchema(respBody.Action),
-		WSSURL:   respBody.WSSURL,
-		Password: respBody.Password,
-	}, resp, nil
+	result.Action = ActionFromSchema(respBody.Action)
+	result.WSSURL = respBody.WSSURL
+	result.Password = respBody.Password
+
+	return result, resp, nil
 }
 
 // ServerAttachToNetworkOpts specifies options for attaching a server to a network.
@@ -993,6 +909,11 @@ type ServerAttachToNetworkOpts struct {
 
 // AttachToNetwork attaches a server to a network.
 func (c *ServerClient) AttachToNetwork(ctx context.Context, server *Server, opts ServerAttachToNetworkOpts) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/attach_to_network"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionAttachToNetworkRequest{
 		Network: opts.Network.ID,
 	}
@@ -1002,22 +923,12 @@ func (c *ServerClient) AttachToNetwork(ctx context.Context, server *Server, opts
 	for _, aliasIP := range opts.AliasIPs {
 		reqBody.AliasIPs = append(reqBody.AliasIPs, Ptr(aliasIP.String()))
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/attach_to_network", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionAttachToNetworkResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionAttachToNetworkResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, err
 }
 
@@ -1028,25 +939,20 @@ type ServerDetachFromNetworkOpts struct {
 
 // DetachFromNetwork detaches a server from a network.
 func (c *ServerClient) DetachFromNetwork(ctx context.Context, server *Server, opts ServerDetachFromNetworkOpts) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/detach_from_network"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionDetachFromNetworkRequest{
 		Network: opts.Network.ID,
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
 
-	path := fmt.Sprintf("/servers/%d/actions/detach_from_network", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
-
-	respBody := schema.ServerActionDetachFromNetworkResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionDetachFromNetworkResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, err
 }
 
@@ -1058,6 +964,11 @@ type ServerChangeAliasIPsOpts struct {
 
 // ChangeAliasIPs changes a server's alias IPs in a network.
 func (c *ServerClient) ChangeAliasIPs(ctx context.Context, server *Server, opts ServerChangeAliasIPsOpts) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/change_alias_ips"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionChangeAliasIPsRequest{
 		Network:  opts.Network.ID,
 		AliasIPs: []string{},
@@ -1065,21 +976,12 @@ func (c *ServerClient) ChangeAliasIPs(ctx context.Context, server *Server, opts
 	for _, aliasIP := range opts.AliasIPs {
 		reqBody.AliasIPs = append(reqBody.AliasIPs, aliasIP.String())
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
-	path := fmt.Sprintf("/servers/%d/actions/change_alias_ips", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
 
-	respBody := schema.ServerActionDetachFromNetworkResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionChangeAliasIPsResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
+
 	return ActionFromSchema(respBody.Action), resp, err
 }
 
@@ -1102,32 +1004,34 @@ type ServerGetMetricsOpts struct {
 	Step  int
 }
 
-func (o *ServerGetMetricsOpts) addQueryParams(req *http.Request) error {
-	query := req.URL.Query()
-
+func (o ServerGetMetricsOpts) Validate() error {
 	if len(o.Types) == 0 {
-		return fmt.Errorf("no metric types specified")
+		return missingField(o, "Types")
 	}
+	if o.Start.IsZero() {
+		return missingField(o, "Start")
+	}
+	if o.End.IsZero() {
+		return missingField(o, "End")
+	}
+	return nil
+}
+
+func (o ServerGetMetricsOpts) values() url.Values {
+	query := url.Values{}
+
 	for _, typ := range o.Types {
 		query.Add("type", string(typ))
 	}
 
-	if o.Start.IsZero() {
-		return fmt.Errorf("no start time specified")
-	}
 	query.Add("start", o.Start.Format(time.RFC3339))
-
-	if o.End.IsZero() {
-		return fmt.Errorf("no end time specified")
-	}
 	query.Add("end", o.End.Format(time.RFC3339))
 
 	if o.Step > 0 {
 		query.Add("step", strconv.Itoa(o.Step))
 	}
-	req.URL.RawQuery = query.Encode()
 
-	return nil
+	return query
 }
 
 // ServerMetrics contains the metrics requested for a Server.
@@ -1146,64 +1050,60 @@ type ServerMetricsValue struct {
 
 // GetMetrics obtains metrics for Server.
 func (c *ServerClient) GetMetrics(ctx context.Context, server *Server, opts ServerGetMetricsOpts) (*ServerMetrics, *Response, error) {
-	var respBody schema.ServerGetMetricsResponse
+	const opPath = "/servers/%d/metrics?%s"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
 	if server == nil {
-		return nil, nil, fmt.Errorf("illegal argument: server is nil")
+		return nil, nil, missingArgument(server)
 	}
 
-	path := fmt.Sprintf("/servers/%d/metrics", server.ID)
-	req, err := c.client.NewRequest(ctx, "GET", path, nil)
+	if err := opts.Validate(); err != nil {
+		return nil, nil, err
+	}
+
+	reqPath := fmt.Sprintf(opPath, server.ID, opts.values().Encode())
+
+	respBody, resp, err := getRequest[schema.ServerGetMetricsResponse](ctx, c.client, reqPath)
 	if err != nil {
-		return nil, nil, fmt.Errorf("new request: %v", err)
+		return nil, resp, err
 	}
-	if err := opts.addQueryParams(req); err != nil {
-		return nil, nil, fmt.Errorf("add query params: %v", err)
-	}
-	resp, err := c.client.Do(req, &respBody)
+
+	metrics, err := serverMetricsFromSchema(&respBody)
 	if err != nil {
-		return nil, nil, fmt.Errorf("get metrics: %v", err)
+		return nil, nil, fmt.Errorf("convert response body: %w", err)
 	}
-	ms, err := serverMetricsFromSchema(&respBody)
-	if err != nil {
-		return nil, nil, fmt.Errorf("convert response body: %v", err)
-	}
-	return ms, resp, nil
+
+	return metrics, resp, nil
 }
 
 func (c *ServerClient) AddToPlacementGroup(ctx context.Context, server *Server, placementGroup *PlacementGroup) (*Action, *Response, error) {
+	const opPath = "/servers/%d/actions/add_to_placement_group"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
+
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
 	reqBody := schema.ServerActionAddToPlacementGroupRequest{
 		PlacementGroup: placementGroup.ID,
 	}
-	reqBodyData, err := json.Marshal(reqBody)
-	if err != nil {
-		return nil, nil, err
-	}
-	path := fmt.Sprintf("/servers/%d/actions/add_to_placement_group", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, bytes.NewReader(reqBodyData))
-	if err != nil {
-		return nil, nil, err
-	}
 
-	respBody := schema.ServerActionAddToPlacementGroupResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	respBody, resp, err := postRequest[schema.ServerActionAddToPlacementGroupResponse](ctx, c.client, reqPath, reqBody)
 	if err != nil {
 		return nil, resp, err
 	}
-	return ActionFromSchema(respBody.Action), resp, err
+
+	return ActionFromSchema(respBody.Action), resp, nil
 }
 
 func (c *ServerClient) RemoveFromPlacementGroup(ctx context.Context, server *Server) (*Action, *Response, error) {
-	path := fmt.Sprintf("/servers/%d/actions/remove_from_placement_group", server.ID)
-	req, err := c.client.NewRequest(ctx, "POST", path, nil)
-	if err != nil {
-		return nil, nil, err
-	}
+	const opPath = "/servers/%d/actions/remove_from_placement_group"
+	ctx = ctxutil.SetOpPath(ctx, opPath)
 
-	respBody := schema.ServerActionRemoveFromPlacementGroupResponse{}
-	resp, err := c.client.Do(req, &respBody)
+	reqPath := fmt.Sprintf(opPath, server.ID)
+
+	respBody, resp, err := postRequest[schema.ServerActionRemoveFromPlacementGroupResponse](ctx, c.client, reqPath, nil)
 	if err != nil {
 		return nil, resp, err
 	}
-	return ActionFromSchema(respBody.Action), resp, err
+
+	return ActionFromSchema(respBody.Action), resp, nil
 }
